#!usr/bin/perl -w

# test_lg_pairwise_linkage.pl

use strict;
use warnings;
use Getopt::Long;

my ($marker_lgs, $marker_gntps, $target_1, $target_2, $exclude_list, $output);

my $usage = "

Usage: $0

OPTIONS:
--lg_1 [FILE] -----> target linkage group 1
--lg_2 [FILE] -----> target linkage group 2
--output [FILE] ---> output summary of pairwise LD stats for clusters

";

Getopt::Long::GetOptions(
'marker_lgs=s' => \$marker_lgs,
'marker_gnt=s' => \$marker_gntps,
'lg_1=s' => \$target_1,
'lg_2=s' => \$target_2,
'output=s' => \$output
) or die "$usage\n\n";

#if (!defined $marker_lgs) {die "$usage\n";}
#if (!defined $marker_gntps) {die "$usage\n";}
#if (!defined $target_1) {die "designate file containing genotypes for markers in cluster 1\n";}
#if (!defined $target_2) {die "designate file containing genotypes for markers in cluster 2\n";}
#if (!-e $target_1) {die "file containing genotypes for markers in cluster 1 does not exist\n";}
#if (!-e $target_2) {die "file containing genotypes for markers in cluster 2 does not exist\n";}
#if (!defined $output) {die "designate output file\n";}


################################################################################
### SAVE MARKER LINKAGE GROUP ASSIGNMENTS
################################################################################

my %marker_2_lg;
my %lg_2_marker;

open (my $read_lgs, '<', 'ld_comp.marker_lgs.txt') or die "could not open input file\n";
while (my $line = <$read_lgs>) {
	chomp $line;
	my ($marker, $lg) = split /\s+/, $line;
	$marker_2_lg{$marker} = $lg;
	$lg_2_marker{$lg} = $marker;
}
close $read_lgs;

################################################################################
### SAVE CLUSTER1 AND CLUSTER2 MARKER GENOTYPES
################################################################################

my %marker_type;
my %marker_gntp;

my @cluster1_markers;
my %cluster1_chrcounts;
my $c1_count = 0;

my @cluster2_markers;
my %cluster2_chrcounts;
my $c2_count = 0;

open (my $read_gntps, '<', 'ld_comp.marker_gntps.txt') or die "could not open input file\n";
while (my $line = <$read_gntps>) {
	if ($line !~ /^\*/) {next;}
	chomp $line;
	my @line_sections = split /\t/, $line;
	my $bin = shift @line_sections;
	$bin =~ s/^\*//g;
	#warn "assessing marker $bin\n";
	my $class = shift @line_sections;
	$marker_type{$bin} = $class;
	my $genotype = join ";", @line_sections;
	$genotype =~ s/ab/1/g;
	$genotype =~ s/a/0/g;
	$genotype =~ s/b/2/g;
	if (!exists $marker_2_lg{$bin}) {
		#warn "no LG saved for marker $bin\n";
		next;
	}
	elsif ($marker_2_lg{$bin} eq $target_1) {
		push @cluster1_markers, $bin;
		$marker_gntp{$bin} = $genotype;
		my ($bin_chr, undef) = split /_/, $bin;
		$cluster1_chrcounts{$bin_chr} += 1;
		$c1_count++;
	}
	elsif ($marker_2_lg{$bin} eq $target_2) {
		push @cluster2_markers, $bin;
		$marker_gntp{$bin} = $genotype;
		my ($bin_chr, undef) = split /_/, $bin;
		$cluster2_chrcounts{$bin_chr} += 1;
		$c2_count++;
	}
	else {next;}
}

warn "saved $c1_count lg1 markers\n";
warn "saved $c2_count lg2 markers\n";

################################################################################
### SAVE CLUSTER1 AND CLUSTER2 MARKER DOMINANT CHR
################################################################################

my $c1_max_percent = 0;
my $c1_max_chr;
foreach my $c1_check_binchr (keys %cluster1_chrcounts) {
	my $chr_percent = sprintf("%.2f", (100 * ($cluster1_chrcounts{$c1_check_binchr} / scalar(@cluster1_markers))));
	if ($chr_percent > $c1_max_percent) {
		$c1_max_percent = $chr_percent;
		$c1_max_chr = $c1_check_binchr;
	}
}

my $c2_max_percent = 0;
my $c2_max_chr;
foreach my $c2_check_binchr (keys %cluster2_chrcounts) {
	my $chr_percent = sprintf("%.2f", (100 * ($cluster2_chrcounts{$c2_check_binchr} / scalar(@cluster2_markers))));
	if ($chr_percent > $c2_max_percent) {
		$c2_max_percent = $chr_percent;
		$c2_max_chr = $c2_check_binchr;
	}
}


################################################################################
### CALCULATE PAIRWISE LD VALUES FOR MARKER CLUSTERS
################################################################################

my @pairwise_ld_values;
my @identical_list;

foreach my $c1_mark (@cluster1_markers) {
	my @cluster1_genotypes = split /;/, "$marker_gntp{$c1_mark}";
	my @cluster2_genotypes;
	foreach my $c2_mark (@cluster2_markers) {
		# skip if comparing het x het/het x hom marker classes
		if ($marker_type{$c1_mark} ne $marker_type{$c2_mark}) {next;}
		### calculate pairwise LD
		@cluster2_genotypes = split /;/, "$marker_gntp{$c2_mark}";
		my $ld;
		# establish that same number of genotypes have been saved
		if (scalar(@cluster1_genotypes) != scalar(@cluster2_genotypes)) {die "genotype array sizes did not match\n";}
		# establish intersection of array elements with no missing data
		my @nomiss_1;
		my @nomiss_2;
		for (my $y = 0; $y <= $#cluster1_genotypes; $y++) {
			if (("$cluster1_genotypes[$y]" ne '-') && ("$cluster2_genotypes[$y]" ne '-')) {
				push @nomiss_1, $cluster1_genotypes[$y];
				push @nomiss_2, $cluster2_genotypes[$y];
			}
		}
		# if no-missing data-intersection of arrays is <60% original array size, set LD to missing data
		if (scalar(@nomiss_1) < (0.82 * scalar(@cluster1_genotypes))) {next;}
		# else calculate LD using subsection of genotypes without missing data
		else {
			my $nomiss_1_ref = \@nomiss_1;
			my $nomiss_2_ref = \@nomiss_2;
			$ld = sprintf("%.3f", correlation($nomiss_1_ref,$nomiss_2_ref));
			# save LD value to 
			push @pairwise_ld_values, abs($ld);
			if ($ld >= 0.99) {
				push @identical_list, "$c1_mark".'/'."$c2_mark";
			}
		}
	}
}



################################################################################
### CALCULATE AND REPORT PAIRWISE LD STATS FOR THESE MARKER CLUSTERS
################################################################################

my $identicals = join ";", @identical_list;

warn "estimating max LD stats\n";

open (my $outfile, '>', $output) or die "could not open output file\n";

my @pairwiseLD_sorted = sort { $b <=> $a } @pairwise_ld_values;

my $mean_top1;
my $mean_top5;
my $mean_top10;
my $mean_top50;
my $mean_top100;
my $mean_top200;
my $mean_top500;
my $mean_top2500;

#warn "sorted pairwise LD values:\n@pairwiseLD_sorted\n";

# calculate mean, top 10 LD values
if (scalar(@pairwiseLD_sorted) >= 1) {
	my @subset = $pairwiseLD_sorted[0];
	$mean_top1 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top1 = 'NA';}
# calculate mean, top 10 LD values
if (scalar(@pairwiseLD_sorted) >= 5) {
	my @subset = @pairwiseLD_sorted[0..4];
	$mean_top5 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top5 = 'NA';}
# calculate mean, top 10 LD values
if (scalar(@pairwiseLD_sorted) >= 10) {
	my @subset = @pairwiseLD_sorted[0..9];
	$mean_top10 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top10 = 'NA';}
# calculate mean, top 50 LD values
if (scalar(@pairwiseLD_sorted) >= 50) {
	my @subset = @pairwiseLD_sorted[0..49];
	$mean_top50 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top50 = 'NA';}
# calculate mean, top 100 LD values
if (scalar(@pairwiseLD_sorted) >= 100) {
	my @subset = @pairwiseLD_sorted[0..99];
	$mean_top100 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top100 = 'NA';}
# calculate mean, top 200 LD values
if (scalar(@pairwiseLD_sorted) >= 200) {
	my @subset = @pairwiseLD_sorted[0..199];
	$mean_top200 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top200 = 'NA';}
# calculate mean, top 500 LD values
if (scalar(@pairwiseLD_sorted) >= 500) {
	my @subset = @pairwiseLD_sorted[0..499];
	$mean_top500 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top500 = 'NA';}
# calculate mean, top 2500 LD values
if (scalar(@pairwiseLD_sorted) >= 2500) {
	my @subset = @pairwiseLD_sorted[0..2499];
	$mean_top2500 = sprintf "%.2f", arraymean(\@subset);
}
else {$mean_top2500 = 'NA';}

# print linkage groups' pairwise LD summary to output
print $outfile "$target_1\t$c1_max_chr-$c1_max_percent\t$c1_count\t$target_2\t$c2_max_chr-$c2_max_percent\t$c2_count\t$mean_top1\t$mean_top5\t$mean_top10\t$mean_top50\t$mean_top100\t$mean_top200\t$mean_top500\t$mean_top2500\t$identicals\n";

close $outfile;



################################################
# SUBROUTINES
################################################

##########################################################################################
### array mean

sub arraymean {
	my ($arrayref) = @_;
	my $result;
	foreach (@$arrayref) { $result += $_; }
	return $result / @$arrayref;
}

##########################################################################################
### Michael's array median
sub median {
	my @vals = sort {$a <=> $b} @_;
	my $len = @vals;
	if($len%2) {return $vals[int($len/2)];}
	else {return ($vals[int($len/2)-1] + $vals[int($len/2)])/2;}
}

##########################################################################################
### Correlation

sub correlation {

	my ($Corr_arrayA_ref, $Corr_arrayB_ref) = @_;

	my $mean_arrayA = &Mean($Corr_arrayA_ref);
	my $mean_arrayB = &Mean($Corr_arrayB_ref);

	my $ssxx = &SS($Corr_arrayA_ref,$Corr_arrayA_ref,$mean_arrayA,$mean_arrayA);
	my $ssyy = &SS($Corr_arrayB_ref,$Corr_arrayB_ref,$mean_arrayB,$mean_arrayB);
	my $ssxy = &SS($Corr_arrayA_ref,$Corr_arrayB_ref,$mean_arrayA,$mean_arrayB);

	my $correlelation;
	if ($ssxy != 0) {
		$correlelation = &Correl($ssxx, $ssyy, $ssxy);
	}
	else {$correlelation = 0;}

	my $xcorrelation = sprintf("%.3f", $correlelation);

	return $xcorrelation;

}  # End of Correlation


##########################################################################################
### Mean = calculate mean of both arrays

sub Mean {

	my ($Mean_arrayA_ref) = @_;
	my $Mean_nbdata = scalar @$Mean_arrayA_ref;
	my $sum = 0;

	for (my $i = 0; $i <= ($Mean_nbdata - 1); $i++){
		$sum += ${ $Mean_arrayA_ref }[$i];
	}

	my $mu = ($sum / $Mean_nbdata);

	return $mu;

}

##########################################################################################
### SS = sum of squared deviations to the mean

sub SS {

	my ($SS_arrayA_ref, $SS_arrayB_ref, $mean_ArrayA, $mean_ArrayB) = @_;
	my $SS_nbdata = scalar @$SS_arrayA_ref;
	my $sum = 0;
	my ($arrayA_value, $arrayB_value);

	for (my $i = 0; $i <= ($SS_nbdata - 1); $i++){
		#$sum=$sum+($x[$a][$i]-$mean[$a])*($x[$b][$i]-$mean[$b]);
		$arrayA_value = ${ $SS_arrayA_ref }[$i];
		$arrayB_value = ${ $SS_arrayB_ref }[$i];
		$sum += (($arrayA_value - $mean_ArrayA) * ($arrayB_value - $mean_ArrayB));
	}

	return $sum;

}

##########################################################################################
### Correlation

sub Correl {

	my ($ssxx, $ssyy, $ssxy) = @_;

	my $sign = $ssxy / abs($ssxy);

	my $correl = $sign * sqrt($ssxy*$ssxy/($ssxx*$ssyy));

	return $correl;

}


