#!usr/bin/perl -w

use strict;
use warnings;


############################################################
### read marker genotypes and population header info
############################################################
my $cross_head;
my $samp_head;
my $samp_n;
my $line_count = 0;
my %marker_entries;
open (my $read_gntps, '<', 'ld_comp.marker_gntps.txt') or die "could not open input file, make sure ld_comp.marker_gntps.txt exists\n";
while (my $line = <$read_gntps>) {
	chomp $line;
	$line_count++;
	if ($line_count == 1) {$cross_head = $line;}
	elsif ($line_count == 2) {
		my @line_sections = split /\s+/, $line;
		$samp_n = $line_sections[0];
	}
	elsif ($line_count == 3) {$samp_head = $line;}
	else {
		if ($line !~ /^\*/) {next;}
		my @line_sections = split /\s+/, $line;
		my $marker = $line_sections[0];
		$marker =~ s/^\*//g;
		$marker_entries{$marker} = $line;
	}
}
close $read_gntps;


############################################################
### link markers to chromosome through subgroups assignments
############################################################

my %chrom_subgroups;
my %subgroup_markers;
my @chrom_list;

### save list of markers associated with each subgroup
my %marker_2_subgroup;
open (my $marker_subgroups, '<', 'ld_comp.marker_lgs.txt') or die "could not open input file\n";
while (my $line = <$marker_subgroups>) {
	chomp $line;
	my ($marker, $subgroup) = split /\t/, $line;
	if (!exists $subgroup_markers{$subgroup}) {$subgroup_markers{$subgroup} = "$marker";}
	else {$subgroup_markers{$subgroup} = "$subgroup_markers{$subgroup}".';'."$marker";}
}
close $marker_subgroups;

### save list of subgroups associated with each chromosome
open (my $subgroup_chrs, '<', 'ld_comp.lg_chroms.txt') or die "could not open input file\n";
while (my $line = <$subgroup_chrs>) {
	chomp $line;
	my ($subgroup, $chrom) = split /\t/, $line;
	if (!exists $chrom_subgroups{$chrom}) {$chrom_subgroups{$chrom} = "$subgroup";}
	else {$chrom_subgroups{$chrom} = "$chrom_subgroups{$chrom}".';'."$subgroup";}
}
close $subgroup_chrs;


############################################################
### print chromosome-specific marker output to separate onemap files
############################################################
foreach my $print_chr (@chrom_list) {
	open (my $outfile, '>', 'genotypes.'."$print_chr".'.onemap.txt') or die "could not open output file\n";
	my @chrom_subgroup_list = split /;/, $chrom_subgroups{$print_chr};
	my $marker_n = 0;
	### pass 1 - get number of output markers for header
	foreach my $subgroup_check1 (@chrom_subgroup_list) {
		my @subgroup_marker_list = split /;/, $subgroup_markers{$subgroup_check1};
		$marker_n += scalar(@subgroup_marker_list);
	}
	### pass 2 - print onemap output file
	print $outfile "$cross_head\n";
	print $outfile "$samp_n $marker_n 0 0 0\n";
	print $outfile "$samp_head\n";
	foreach my $subgroup_check2 (@chrom_subgroup_list) {
		my @subgroup_marker_list = split /;/, $subgroup_markers{$subgroup_check2};
		foreach my $print_marker (@subgroup_marker_list) {print $outfile "$marker_entries{$print_marker}\n";}
	}
	close $outfile;
}



